"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const colors_1 = tslib_1.__importDefault(require("colors"));
const fs = tslib_1.__importStar(require("fs"));
const fs_extra_1 = require("fs-extra");
const inquirer_1 = require("inquirer");
const ora_1 = tslib_1.__importDefault(require("ora"));
const path_1 = require("path");
const rimraf = tslib_1.__importStar(require("rimraf"));
const zip_a_folder_1 = require("zip-a-folder");
const botonic_api_service_1 = require("../botonic-api-service");
const utils_1 = require("../utils");
let force = false;
let npmCommand;
const BOTONIC_BUNDLE_FILE = 'botonic_bundle.zip';
class Run extends command_1.Command {
    constructor() {
        super(...arguments);
        this.botonicApiService = new botonic_api_service_1.BotonicAPIService();
        this.botName = undefined;
    }
    async run() {
        const { flags } = this.parse(Run);
        utils_1.track('Deployed Botonic CLI');
        force = flags.force || false;
        npmCommand = flags.command;
        this.botName = flags.botName;
        const email = flags.email;
        const password = flags.password;
        if (email && password)
            await this.login(email, password);
        else if (!this.botonicApiService.oauth)
            await this.signupFlow();
        else if (this.botName) {
            await this.deployBotFromFlag(this.botName);
        }
        else
            await this.deployBotFlow();
    }
    async deployBotFromFlag(botName) {
        const resp = await this.botonicApiService.getBots();
        const nextBots = resp.data.next;
        const bots = resp.data.results;
        if (nextBots) {
            await this.botonicApiService.getMoreBots(bots, nextBots);
        }
        const bot = bots.filter(b => b.name === botName)[0];
        if (bot == undefined) {
            console.log(colors_1.default.red(`Bot ${botName} doesn't exist.`));
            console.log('\nThese are the available options:');
            bots.map(b => console.log(` > ${b.name}`));
        }
        else {
            this.botonicApiService.setCurrentBot(bot);
            await this.deploy();
        }
    }
    signupFlow() {
        const choices = [
            'No, I need to create a new one (Signup)',
            'Yes, I do. (Login)',
        ];
        return inquirer_1.prompt([
            {
                type: 'list',
                name: 'signupConfirmation',
                message: 'You need to login before deploying your bot.\nDo you have a Hubtype account already?',
                choices: choices,
            },
        ]).then((inp) => {
            if (inp.signupConfirmation == choices[1])
                return this.askLogin();
            else
                return this.askSignup();
        });
    }
    askEmailPassword() {
        return inquirer_1.prompt([
            {
                type: 'input',
                name: 'email',
                message: 'email:',
            },
            {
                type: 'password',
                name: 'password',
                mask: '*',
                message: 'password:',
            },
        ]);
    }
    async askLogin() {
        await this.askEmailPassword().then(inp => this.login(inp.email, inp.password));
    }
    async askSignup() {
        await this.askEmailPassword().then(inp => this.signup(inp.email, inp.password));
    }
    async deployBotFlow() {
        if (this.botName)
            return this.deployBotFromFlag(this.botName);
        if (!this.botonicApiService.bot)
            return this.newBotFlow();
        else {
            const resp = await this.botonicApiService.getBots();
            const nextBots = resp.data.next;
            const bots = resp.data.results;
            if (nextBots) {
                await this.botonicApiService.getMoreBots(bots, nextBots);
            }
            // Show the current bot in credentials at top of the list
            const first_id = this.botonicApiService.bot.id;
            bots.sort(function (x, y) {
                return x.id == first_id ? -1 : y.id == first_id ? 1 : 0;
            });
            return this.selectExistentBot(bots);
        }
    }
    async login(email, password) {
        return this.botonicApiService.login(email, password).then(({}) => this.deployBotFlow(), async (err) => {
            if (err.response &&
                err.response.data &&
                err.response.data.error_description) {
                console.log(colors_1.default.red(err.response.data.error_description));
            }
            else {
                console.log(colors_1.default.red('There was an error when trying to log in. Please, try again:'));
                if (err.response && err.response.status && err.response.statusText) {
                    console.error(`Error ${err.response.status}: ${err.response.statusText}`);
                }
            }
            await this.askLogin();
        });
    }
    async signup(email, password) {
        const org_name = email.split('@')[0];
        const campaign = { product: 'botonic' };
        return this.botonicApiService
            .signup(email, password, org_name, campaign)
            .then(({}) => this.login(email, password), async (err) => {
            if (err.response.data.email)
                console.log(colors_1.default.red(err.response.data.email[0]));
            if (err.response.data.password)
                console.log(colors_1.default.red(err.response.data.password[0]));
            if (!err.response.data.email && !err.response.data.password)
                console.log(colors_1.default.red('There was an error trying to signup. Please, try again:'));
            await this.askSignup();
        });
    }
    async newBotFlow() {
        const resp = await this.botonicApiService.getBots();
        const nextBots = resp.data.next;
        const bots = resp.data.results;
        if (nextBots) {
            const _new_bots = await this.botonicApiService.getMoreBots(bots, nextBots);
        }
        if (!bots.length) {
            return this.createNewBot();
        }
        else {
            return inquirer_1.prompt([
                {
                    type: 'confirm',
                    name: 'create_bot_confirm',
                    message: 'Do you want to create a new Bot?',
                },
            ]).then((res) => {
                const confirm = res.create_bot_confirm;
                if (confirm) {
                    return this.createNewBot();
                }
                else {
                    return this.selectExistentBot(bots);
                }
            });
        }
    }
    createNewBot() {
        return inquirer_1.prompt([
            {
                type: 'input',
                name: 'bot_name',
                message: 'Bot name:',
            },
        ]).then((inp) => {
            return this.botonicApiService.saveBot(inp.bot_name).then(({}) => this.deploy(), err => console.log(colors_1.default.red(`There was an error saving the bot: ${err}`)));
        });
    }
    selectExistentBot(bots) {
        return inquirer_1.prompt([
            {
                type: 'list',
                name: 'bot_name',
                message: 'Please, select a bot',
                choices: bots.map(b => b.name),
            },
        ]).then((inp) => {
            const bot = bots.filter(b => b.name === inp.bot_name)[0];
            this.botonicApiService.setCurrentBot(bot);
            return this.deploy();
        });
    }
    displayProviders(providers) {
        console.log('Your bot is published on:');
        providers.forEach(p => {
            if (p.provider === 'whatsapp')
                console.log(`💬  [whatsapp] https://wa.me/${p.username}`);
            if (p.provider === 'facebook')
                console.log(`💬  [facebook] https://m.me/${p.username}`);
            if (p.provider === 'telegram')
                console.log(`💬  [telegram] https://t.me/${p.username}`);
            if (p.provider === 'twitter')
                console.log(`💬  [twitter] https://t.me/${p.username}`);
            if (p.provider === 'generic')
                console.log(`💬  Your app or website`);
        });
    }
    async createBundle() {
        const spinner = ora_1.default({
            text: 'Creating bundle...',
            spinner: 'bouncingBar',
        }).start();
        // Why using rimraf instead of fs.rmdirSync?: https://stackoverflow.com/a/57834155/145289
        if (fs.existsSync('tmp'))
            rimraf.sync('tmp');
        fs.mkdirSync(path_1.join('tmp'));
        fs_extra_1.copySync('dist', path_1.join('tmp', 'dist'));
        const zipRes = await zip_a_folder_1.zip('tmp', path_1.join(BOTONIC_BUNDLE_FILE));
        if (zipRes instanceof Error) {
            throw Error;
        }
        const zip_stats = fs.statSync(BOTONIC_BUNDLE_FILE);
        spinner.succeed();
        if (zip_stats.size >= 10 * 10 ** 6) {
            spinner.fail();
            console.log(colors_1.default.red(`Deploy failed. Bundle size too big ${zip_stats.size} (max 10Mb).`));
            utils_1.track('Deploy Botonic Zip Error');
            return;
        }
    }
    async deployBundle() {
        const spinner = ora_1.default({
            text: 'Deploying...',
            spinner: 'bouncingBar',
        }).start();
        try {
            const deploy = await this.botonicApiService.deployBot(path_1.join(process.cwd(), BOTONIC_BUNDLE_FILE), force);
            if ((deploy.response && deploy.response.status == 403) ||
                !deploy.data.deploy_id) {
                utils_1.track('Deploy Botonic Error', { error: deploy.response.data.status });
                throw deploy.response.data.status;
            }
            // eslint-disable-next-line no-constant-condition
            while (true) {
                await utils_1.sleep(500);
                const deploy_status = await this.botonicApiService.deployStatus(deploy.data.deploy_id);
                if (deploy_status.data.is_completed) {
                    if (deploy_status.data.status == 'deploy_status_completed_ok') {
                        spinner.succeed();
                        console.log(colors_1.default.green('\n🚀  Bot deployed!\n'));
                        break;
                    }
                    else {
                        spinner.fail();
                        console.log(colors_1.default.red('There was a problem in the deploy:'));
                        console.log(deploy_status.data.error);
                        utils_1.track('Deploy Botonic Error', { error: deploy_status.data.error });
                        return;
                    }
                }
            }
        }
        catch (err) {
            spinner.fail();
            console.log(colors_1.default.red('There was a problem in the deploy:'));
            console.log(err);
            utils_1.track('Deploy Botonic Error', { error: err });
            return;
        }
    }
    async displayDeployResults() {
        try {
            const providers_resp = await this.botonicApiService.getProviders();
            const providers = providers_resp.data.results;
            if (!providers.length) {
                const links = `Now, you can integrate a channel in:\nhttps://app.hubtype.com/bots/${this.botonicApiService.bot.id}/integrations?access_token=${this.botonicApiService.oauth.access_token}`;
                console.log(links);
            }
            else {
                this.displayProviders(providers);
            }
        }
        catch (e) {
            utils_1.track('Deploy Botonic Provider Error', { error: e });
            console.log(colors_1.default.red(`There was an error getting the providers: ${e}`));
        }
    }
    async deploy() {
        try {
            const build_out = await this.botonicApiService.buildIfChanged(force, npmCommand);
            if (!build_out) {
                utils_1.track('Deploy Botonic Build Error');
                console.log(colors_1.default.red('There was a problem building the bot'));
                return;
            }
            await this.createBundle();
            await this.deployBundle();
            await this.displayDeployResults();
        }
        catch (e) {
            console.log(colors_1.default.red('Deploy Error'), e);
        }
        finally {
            fs_extra_1.removeSync(BOTONIC_BUNDLE_FILE);
            fs_extra_1.removeSync('tmp');
            this.botonicApiService.beforeExit();
        }
    }
}
exports.default = Run;
Run.description = 'Deploy Botonic project to hubtype.com';
Run.examples = [
    `$ botonic deploy
Building...
Creating bundle...
Uploading...
🚀 Bot deployed!
`,
];
Run.flags = {
    force: command_1.flags.boolean({
        char: 'f',
        description: 'Force deploy despite of no changes. Disabled by default',
    }),
    command: command_1.flags.string({
        char: 'c',
        description: 'Command to execute from the package "scripts" object',
    }),
    email: command_1.flags.string({
        char: 'e',
        description: 'Email from Hubtype Organization',
    }),
    password: command_1.flags.string({
        char: 'p',
        description: 'Password from Hubtype Organization',
    }),
    botName: command_1.flags.string({
        char: 'b',
        description: 'Name of the bot from Hubtype where you want to deploy',
    }),
};
Run.args = [{ name: 'bot_name' }];
//# sourceMappingURL=deploy.js.map