"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer_1 = require("inquirer");
const path_1 = require("path");
const botonic_api_service_1 = require("../botonic-api-service");
const utils_1 = require("../utils");
class Run extends command_1.Command {
    constructor() {
        super(...arguments);
        this.botonicApiService = new botonic_api_service_1.BotonicAPIService();
    }
    async run() {
        utils_1.track('Logged In Botonic CLI');
        const { flags } = this.parse(Run);
        const _path = flags.path ? path_1.resolve(flags.path) : process.cwd();
        await this.logInUser();
    }
    askLoginInfo() {
        return inquirer_1.prompt([
            {
                type: 'input',
                name: 'email',
                message: 'email:',
            },
            {
                type: 'password',
                name: 'password',
                mask: '*',
                message: 'password:',
            },
        ]);
    }
    async logInUser() {
        const user_data = await this.askLoginInfo();
        await this.botonicApiService
            .login(user_data.email, user_data.password)
            .then(() => {
            console.log('Successful log in!'.green);
            return;
        }, err => {
            console.log('Error: '.red, err.response.data.error_description.red);
        });
    }
}
exports.default = Run;
Run.description = 'Log in to Botonic';
Run.examples = [];
Run.flags = {
    path: command_1.flags.string({
        char: 'p',
        description: 'Path to botonic project. Defaults to current dir.',
    }),
};
Run.args = [];
//# sourceMappingURL=login.js.map