"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const child_process_1 = require("child_process");
const colors_1 = tslib_1.__importDefault(require("colors"));
const fetch_repo_dir_1 = tslib_1.__importDefault(require("fetch-repo-dir"));
const fs_1 = require("fs");
const fs_extra_1 = require("fs-extra");
const inquirer_1 = require("inquirer");
const ora_1 = tslib_1.__importDefault(require("ora"));
const path_1 = require("path");
const rimraf_1 = tslib_1.__importDefault(require("rimraf"));
const util_1 = require("util");
const botonic_api_service_1 = require("../botonic-api-service");
const botonic_examples_1 = require("../botonic-examples");
const utils_1 = require("../utils");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const exec = util_1.promisify(child_process_1.exec);
class Run extends command_1.Command {
    constructor() {
        super(...arguments);
        this.examples = botonic_examples_1.EXAMPLES;
        this.botonicApiService = new botonic_api_service_1.BotonicAPIService();
    }
    async run() {
        utils_1.track('Created Botonic Bot CLI');
        const { args } = this.parse(Run);
        const userProjectName = args.name;
        let selectedProjectName = '';
        if (!args.projectName) {
            await this.selectBotName().then(resp => {
                selectedProjectName = this.examples.filter(p => p.description === resp.botName)[0].name;
                return;
            });
        }
        else {
            const botExists = this.examples.filter(p => p.name === args.projectName);
            if (botExists.length) {
                selectedProjectName = args.projectName;
            }
            else {
                const projectNames = this.examples.map(p => p.name);
                console.log(colors_1.default.red(`Example ${args.projectName} does not exist, please choose one of ${projectNames}.`));
                return;
            }
        }
        if (fs_1.existsSync(userProjectName)) {
            rimraf_1.default.sync(userProjectName);
        }
        let spinner = ora_1.default({
            text: 'Downloading files...',
            spinner: 'bouncingBar',
        }).start();
        const selectedProject = this.examples.filter(project => project.name === selectedProjectName)[0];
        try {
            await fetch_repo_dir_1.default({
                src: selectedProject.uri,
                dir: userProjectName,
            });
        }
        catch (e) {
            throw new Error(`Error downloading ${selectedProjectName}: ${String(e)}`);
        }
        spinner.succeed();
        process.chdir(args.name);
        spinner = ora_1.default({
            text: 'Installing dependencies...',
            spinner: 'bouncingBar',
        }).start();
        const dependencyCommand = `npm install`;
        await exec(dependencyCommand);
        spinner.succeed();
        await this.botonicApiService.buildIfChanged(false);
        this.botonicApiService.beforeExit();
        fs_extra_1.moveSync(path_1.join('..', '.botonic.json'), path_1.join(process.cwd(), '.botonic.json'));
        const chdirCmd = colors_1.default.bold(`cd ${args.name}`);
        const trainCmd = selectedProjectName === 'nlu' ? colors_1.default.bold(`\nbotonic train`) : undefined;
        const serveCmd = colors_1.default.bold('botonic serve');
        const deployCmd = colors_1.default.bold('botonic deploy');
        console.log(`\n✨  Bot ${colors_1.default.bold(args.name)} was successfully created!\n\nNext steps:\n${chdirCmd}\n${serveCmd}${trainCmd || ''} (test your bot locally from the browser)\n${deployCmd} (publish your bot to the world!)`);
    }
    selectBotName() {
        return inquirer_1.prompt([
            {
                type: 'list',
                name: 'botName',
                message: 'Select a bot example',
                choices: this.examples.map(p => p.description),
            },
        ]);
    }
}
exports.default = Run;
Run.description = 'Create a new Botonic project';
Run.examples = [
    `$ botonic new test_bot
Creating...
✨ test_bot was successfully created!
`,
];
Run.args = [
    { name: 'name', description: 'name of the bot folder', required: true },
    {
        name: 'projectName',
        description: 'OPTIONAL name of the bot project',
        required: false,
    },
];
//# sourceMappingURL=new.js.map