"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.spawnNpmScript = exports.spawnProcess = exports.getGlobalNodeModulesPath = exports.sleep = exports.botonicPostInstall = exports.track = exports.initializeCredentials = exports.botonic_credentials_path = exports.botonic_home_path = exports.credentials = exports.analytics = void 0;
const tslib_1 = require("tslib");
const analytics_node_1 = tslib_1.__importDefault(require("analytics-node"));
const child_process_1 = require("child_process");
const fs = tslib_1.__importStar(require("fs"));
const os = tslib_1.__importStar(require("os"));
const path = tslib_1.__importStar(require("path"));
exports.botonic_home_path = path.join(os.homedir(), '.botonic');
exports.botonic_credentials_path = path.join(exports.botonic_home_path, 'credentials.json');
function initializeCredentials() {
    if (!fs.existsSync(exports.botonic_home_path))
        fs.mkdirSync(exports.botonic_home_path);
    const anonymous_id = Math.round(Math.random() * 100000000);
    fs.writeFileSync(exports.botonic_credentials_path, JSON.stringify({ analytics: { anonymous_id } }));
}
exports.initializeCredentials = initializeCredentials;
function readCredentials() {
    if (!fs.existsSync(exports.botonic_credentials_path)) {
        initializeCredentials();
    }
    try {
        exports.credentials = JSON.parse(fs.readFileSync(exports.botonic_credentials_path, 'utf8'));
    }
    catch (e) {
        if (fs.existsSync(exports.botonic_credentials_path)) {
            console.warn('Credentials could not be loaded', e);
        }
    }
}
try {
    readCredentials();
}
catch (e) {
    console.warn('Credentials could not be loaded', e);
}
function analytics_enabled() {
    return process.env.BOTONIC_DISABLE_ANALYTICS !== '1';
}
if (analytics_enabled()) {
    exports.analytics = new analytics_node_1.default('YD0jpJHNGW12uhLNbgB4wbdTRQ4Cy1Zu');
}
function getSystemInformation() {
    return {
        platform: os.platform(),
        version: os.version(),
        arch: os.arch(),
        timezone: Intl.DateTimeFormat().resolvedOptions().timeZone,
        timestamp: new Date().toISOString(),
    };
}
function track(event, properties = {}) {
    if (analytics_enabled() &&
        exports.analytics &&
        exports.credentials &&
        exports.credentials.analytics) {
        properties = Object.assign(Object.assign({}, properties), getSystemInformation());
        exports.analytics.track({
            event: event,
            anonymousId: exports.credentials.analytics.anonymous_id,
            properties: properties,
        });
    }
}
exports.track = track;
function botonicPostInstall() {
    if (analytics_enabled()) {
        initializeCredentials();
        readCredentials();
        track('Installed Botonic CLI');
    }
}
exports.botonicPostInstall = botonicPostInstall;
function sleep(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}
exports.sleep = sleep;
async function sh(cmd) {
    return new Promise(function (resolve, reject) {
        child_process_1.exec(cmd, (err, stdout, stderr) => {
            if (err) {
                reject(err);
            }
            else {
                resolve({ stdout, stderr });
            }
        });
    });
}
async function getGlobalNodeModulesPath() {
    const CROSS_PLATFORM_REGEX = /\r?\n|\r/g;
    return (await sh('npm root -g')).stdout.replace(CROSS_PLATFORM_REGEX, '');
}
exports.getGlobalNodeModulesPath = getGlobalNodeModulesPath;
function spawnProcess(command, args, onClose) {
    const childProcess = child_process_1.spawn(command, args, { shell: true }); // https://nodejs.org/api/child_process.html#child_process_spawning_bat_and_cmd_files_on_windows
    childProcess.stdout.on('data', out => {
        process.stdout.write(out);
    });
    childProcess.stderr.on('data', stderr => {
        process.stderr.write(stderr);
    });
    childProcess.on('close', code => {
        onClose && onClose();
        process.stdout.write(`child process exited with code ${code}`);
    });
}
exports.spawnProcess = spawnProcess;
function spawnNpmScript(script, onClose) {
    spawnProcess('npm', ['run', script], onClose);
}
exports.spawnNpmScript = spawnNpmScript;
//# sourceMappingURL=utils.js.map